package com.gic.store.service.outer;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.entity.TabStoreStrategy;
import com.gic.store.service.StoreStrategyApiService;
import com.gic.store.service.StoreStrategyService;
import com.gic.store.utils.ErrorCode;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author guojx
 * @date 2019/7/4 10:10 AM
 */
@Service("storeStrategyApiService")
public class StoreStrategyApiServiceImpl implements StoreStrategyApiService{

    @Autowired
    private StoreStrategyService storeStrategyService;

    @Override
    public ServiceResponse<Integer> saveStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO) {
        if (StringUtils.isBlank(storeStrategyDTO.getStrategyName())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略名称不能为空");
        }
        if (storeStrategyDTO.getStoreRegionId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "门店域不能为空");
        }
        if (storeStrategyDTO.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "企业ID不能为空");
        }
        if (StringUtils.isBlank(storeStrategyDTO.getTargetValue())) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略命中值不能为空");
        }
        //分组策略
        storeStrategyDTO.setStrategyType(StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
        storeStrategyDTO.setCreateTime(new Date());
        storeStrategyDTO.setUpdateTime(new Date());
        storeStrategyDTO.setStatus(1);
        //sort
        storeStrategyDTO.setSort(storeStrategyService.getMinSort(storeStrategyDTO.getEnterpriseId(), storeStrategyDTO.getStrategyType()) - 1d);

        TabStoreStrategy record = EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO);
        if (storeStrategyService.isRepeatGroupStrategyName(record)) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略名称不能重复");
        }

        return  ServiceResponse.success(storeStrategyService.saveStoreGroupStrategy(storeStrategyDTO));
    }

    @Override
    public ServiceResponse<Integer> deleteStoreGroupStrategy(int strategyId) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略主键ID错误，查询不到数据");
        }
        return ServiceResponse.success(storeStrategyService.deleteStoreGroupStrategy(strategyId));
    }

    @Override
    public ServiceResponse<Integer> modifyStoreGroupStrategy(StoreStrategyDTO storeStrategyDTO) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(storeStrategyDTO.getStrategyId());
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyDTO.setCreateTime(storeStrategy.getCreateTime());
        storeStrategyDTO.setUpdateTime(new Date());
        storeStrategyDTO.setEnterpriseId(storeStrategy.getEnterpriseId());
        storeStrategyDTO.setStrategyType(storeStrategy.getStrategyType());

        TabStoreStrategy record = EntityUtil.changeEntityNew(TabStoreStrategy.class, storeStrategyDTO);
        if (storeStrategyService.isRepeatGroupStrategyName(record)) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略名称不能重复");
        }
        return ServiceResponse.success(storeStrategyService.modifyStoreGroupStrategy(storeStrategyDTO));
    }

    @Override
    public ServiceResponse<Integer> sortBottom(int strategyId, int strategyType) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.sortBottom(storeStrategy.getEnterpriseId(), strategyId, strategyType);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> sortTopping(int strategyId, int strategyType) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.sortTopping(storeStrategy.getEnterpriseId(), strategyId, strategyType);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Integer> setStoreStrategySort(int strategyId, int setValue, int strategyType) {
        TabStoreStrategy storeStrategy = storeStrategyService.getStoreStrategyById(strategyId);
        if (storeStrategy == null) {
            return ServiceResponse.failure(ErrorCode.ERR_5.getCode(), "策略主键ID错误，查询不到数据");
        }
        storeStrategyService.setStoreStrategySort(strategyId, storeStrategy.getEnterpriseId(), setValue, strategyType);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<StoreStrategyDTO>> listStoreField(int enterpriseId, String search, int strategyType, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page storeStrategies = storeStrategyService.listStoreStrategy(enterpriseId, search, strategyType, pageNum, pageSize);
        Page<StoreStrategyDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeStrategies, StoreStrategyDTO.class);
        List<StoreStrategyDTO> strategyDTOList = page.getResult();
        int index = (pageNum - 1) * pageSize + 1;
        for (int i = 0, length = strategyDTOList.size(); i < length; i++) {
            strategyDTOList.get(i).setSeq(index + i);
        }
        return ServiceResponse.success(page);
    }
}
